<?php

declare(strict_types=1);

namespace Silex\Router;

use Silex\Http\HttpResponse;
use Silex\DI\DI;

class Router
{

    private string $url;

    /**
     * @var Route[]
     */
    private array $routes = [];

    public function __construct(string $url)
    {
        $url = PathHelper::removeEverythingAfter($url, '?');
        $url = PathHelper::removeEverythingAfter($url, '#');
        $this->url = trim($url, '/');
    }

    public function setBasePath(string $basePath)
    {
        $pos = strpos($this->url, $basePath);
        if ($pos === false) {
            $this->url = null;
        } else {
            $this->url = trim(substr($this->url, $pos + strlen($basePath)), '/');
        }
    }

    public function get(string $path, callable $callable): self
    {
        return $this->addRoute('GET', $path, $callable);
    }

    private function addRoute(string $method, string $path, $callable): self
    {
        $route = new Route($path, $callable);
        $this->routes[$method][] = $route;
        return $this;
    }

    public function run(DI $di): HttpResponse
    {
        if (!isset($this->routes[$_SERVER['REQUEST_METHOD']])) {
            throw new RouteNotFoundException('Unknown HTTP method');
        }
        if ($this->url !== null) {
            foreach ($this->routes[$_SERVER['REQUEST_METHOD']] as $route) {
                if ($route->matches($this->url)) {
                    return $route->call($di);
                }
            }
        }
        throw new RouteNotFoundException('No matching routes');
    }
}
